/***************************************************************************//**
 @file

 @page license License

 @brief Copyright 2009, The DES-SERT Team, Freie Universitaet Berlin (FUB).     \n
 All rights reserved.                                                         \n

 These sources were Bastian Blywis
 at Freie Universitaet Berlin (http://www.fu-berlin.de/),
 Computer Systems and Telematics / Distributed, Embedded Systems (DES) group  \n
 (http://cst.mi.fu-berlin.de/, http://www.des-testbed.net/)                   \n
 -----------------------------------------------------------------------------\n
 This program is free software: you can redistribute it and/or modify it under
 the terms of the GNU General Public License as published by the Free Software
 Foundation, either version 3 of the License, or (at your option) any later
 version.                                                                     \n
 \n
 This program is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.\n
 \n
 You should have received a copy of the GNU General Public License along with
 this program. If not, see http://www.gnu.org/licenses/ .                     \n
 -----------------------------------------------------------------------------\n
 For further information and questions please use the web site                \n
        http://www.des-testbed.net/
*******************************************************************************/

/***************************************************************************//**
 *
 * @mainpage DES-SERT Extra
 *
 *  \image html dessert.png
 *  \image latex dessert.eps width=.7\linewidth
 *
 * @section about_sec About
 *
 * The DES-SERT Extra library (libdessert-extra) contains several sys, mesh, and
 * CLI callback functions that may be used in any DES-SERT based daemon. The
 * source code was part of libdessert but moved to this library so that
 * the libdessert API and ABI can be kept stable.
 *
 * DES-SERT Extra also serves as example for developers how you can accomplish important
 * tasks with simple callbacks.
 *
 * @section des-sert_sec DES-SERT
 *
 * DES-SERT, the DES Simple and Extensible Routing-Framework for Testbeds,
 * is a framework designed to assist researchers in the implementation of
 * routing protocols for testbeds.
 *
 * DES-SERT enables the implementation of routing protocols on top of
 * Ethernet via an underlay (Layer 2.5) in user space; this also includes
 * IEEE 802.11 on the data link layer.
 * It introduces an abstraction from OS specific issues and provides
 * functionality and data structures to implement proactive, reactive,
 * and hybrid routing protocols.

 * While generally usable in many application scenarios, it is primarily
 * used in DES-Mesh (http://www.des-testbed.net/), the multi-transceiver
 * wireless mesh network testbed part of the DES-Testbed at Freie
 * Universitaet Berlin, Germany.
 *
 * @section dessert_sec Content of the Documentation
 *
 * The DES-SERT Extra documentation is split in three parts:
 * - \ref manual
 * - \ref callbacks
 * - \ref helpers
 *
 * The \ref manual section provides sub-categories that discuss particular
 * topics and present corresponding callbacks in the library:
 * - \ref interfaces
 * - \ref filtering
 * - \ref traceroute
 * - \ref ping
 * - \ref trace
 *
 * The \ref callbacks section is divided in three sub-categories and
 * provides the API documentation:
 * - \ref cli
 * - \ref mesh
 * - \ref sys
 *
 *
 * @section feedback_sec Contact & Feedback
 *
 * We love feedback - if you have patches, comments or questions,
 * please contact us! Recent contact information is available on
 *         http://www.des-testbed.net/des-sert/
 ******************************************************************************/

#include <dessert.h>
#include <libcli.h>

#ifndef DESSERT_EXTRA_H
#define DESSERT_EXTRA_H

/***************************************************************************//**
 * @ingroup callbacks
 * @defgroup cli CLI Callbacks
 *
 * @brief CLI Callbacks
 *
 * @{
 ******************************************************************************/

int dessert_cli_cmd_addsysif(struct cli_def *cli, char *command, char *argv[], int argc);
int dessert_cli_cmd_addsysif_tun(struct cli_def *cli, char *command, char *argv[], int argc);
int dessert_cli_cmd_addmeshif(struct cli_def *cli, char *command, char *argv[], int argc);
int dessert_cli_cmd_ping(struct cli_def *cli, char *command, char *argv[], int argc);
int dessert_cli_cmd_traceroute(struct cli_def *cli, char *command, char *argv[], int argc);

/***************************************************************************//**
 * @}
 * @ingroup callbacks
 * @defgroup mesh mesh Callbacks
 *
 * @brief mesh Callbacks
 *
 * @{
 ******************************************************************************/

int dessert_mesh_ping(dessert_msg_t* msg, size_t len, dessert_msg_proc_t *proc, const dessert_meshif_t *iface, dessert_frameid_t id);
int dessert_mesh_pong(dessert_msg_t* msg, size_t len, dessert_msg_proc_t *proc, const dessert_meshif_t *iface, dessert_frameid_t id);
int dessert_mesh_trace(dessert_msg_t* msg, size_t len, dessert_msg_proc_t *proc, const dessert_meshif_t *iface, dessert_frameid_t id);
int dessert_mesh_ipttl(dessert_msg_t* msg, size_t len, dessert_msg_proc_t *proc, const dessert_meshif_t *iface, dessert_frameid_t id);
int dessert_mesh_drop_ethernet(dessert_msg_t* msg, size_t len, dessert_msg_proc_t *proc, const dessert_meshif_t *iface, dessert_frameid_t id);
int dessert_mesh_drop_ip(dessert_msg_t* msg, size_t len, dessert_msg_proc_t *proc, const dessert_meshif_t *iface, dessert_frameid_t id);

/***************************************************************************//**
 * @}
 * @ingroup helpers
 *
 * @brief Helper Functions
 *
 * @{
 ******************************************************************************/

int dessert_msg_trace_initiate(dessert_msg_t* msg, uint8_t type, int mode);

/***************************************************************************//**
 * @}
 * @ingroup callbacks
 * @defgroup sys sys Callbacks
 *
 * @brief sys Callbacks
 *
 * @{
 ******************************************************************************/
 
int dessert_sys_drop_ipv6(dessert_msg_t* msg, size_t len, dessert_msg_proc_t *proc, const dessert_sysif_t *sysif, dessert_frameid_t id);

/** @} */

#endif